@@ -1,6 +1,9 @@
-from .task import Task
+from typing import Mapping
+
 from ewoksutils.import_utils import import_method
 
+from .task import Task
+
 METHOD_ARGUMENT = "_method"
 
 
@@ -9,6 +12,9 @@ class MethodExecutorTask(
 ):
     METHOD_ARGUMENT = METHOD_ARGUMENT
 
+    def _get_task_identifier(self, inputs: Mapping) -> str:
+        return inputs.get(self.METHOD_ARGUMENT, self.class_registry_name())
+
     def run(self):
         kwargs = self.get_named_input_values()
         args = self.get_positional_input_values()
