@@ -1,6 +1,9 @@
-from .task import Task
+from typing import Mapping
+
 from ewoksutils.import_utils import import_method
 
+from .task import Task
+
 METHOD_ARGUMENT = "_method"
 PPF_DICT_ARGUMENT = "_ppfdict"
 
@@ -19,6 +22,9 @@ class PpfMethodExecutorTask(
     METHOD_ARGUMENT = METHOD_ARGUMENT
     PPF_DICT_ARGUMENT = PPF_DICT_ARGUMENT
 
+    def _get_task_identifier(self, inputs: Mapping) -> str:
+        return inputs.get(self.METHOD_ARGUMENT, self.class_registry_name())
+
     def run(self):
         method_kwargs = self.get_input_values()
         fullname = method_kwargs.pop(self.METHOD_ARGUMENT)
@@ -40,6 +46,9 @@ class PpfPortTask(
 
     PPF_DICT_ARGUMENT = PPF_DICT_ARGUMENT
 
+    def _get_task_identifier(self, inputs: Mapping) -> str:
+        return ""
+
     def run(self):
         method_kwargs = self.get_input_values()
         ppfdict = method_kwargs.pop(self.PPF_DICT_ARGUMENT, None)
